package com.aihuishou.picpicker

import android.app.Activity
import androidx.fragment.app.Fragment
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleObserver
import androidx.lifecycle.OnLifecycleEvent
import com.aihuishou.picpicker.tools.GlideEngine
import com.luck.picture.lib.PictureSelectionModel
import com.luck.picture.lib.PictureSelector
import com.luck.picture.lib.animators.AnimationType
import com.luck.picture.lib.config.PictureConfig
import com.luck.picture.lib.config.PictureMimeType
import com.luck.picture.lib.entity.LocalMedia
import com.luck.picture.lib.language.LanguageConfig
import com.luck.picture.lib.listener.OnResultCallbackListener
import com.luck.picture.lib.manager.PictureCacheManager
import com.luck.picture.lib.style.PictureSelectorUIStyle
import com.luck.picture.lib.style.PictureWindowAnimationStyle
import com.yalantis.ucrop.view.OverlayView
import com.zhaoshuang.weixinrecorded.R
import java.lang.ref.WeakReference

/**
 * created by limian
 */

class MediaPicker : LifecycleObserver {

    private var weakActivity: WeakReference<Activity?>? = null
    private var weakFragment: WeakReference<Fragment?>? = null
    private var pictureSelector: PictureSelector? = null
    private var selectionModel: PictureSelectionModel? = null
    private var config: Config? = null

    private var resultCallback: OnResultCallbackListener<LocalMedia>? = null
    private var callback: OnPickResultCallback? = null
    private var simpleCallback: OnSimplePickResultCallback? = null

    private val selectedData = HashSet<LocalMedia>()

    constructor(activity: Activity) {
        if (weakActivity?.get() === activity) {
            return
        }
        weakActivity = WeakReference(activity)
        pictureSelector = PictureSelector.create(activity)
    }

    constructor(fragment: Fragment) {
        if (weakFragment?.get() === fragment) {
            return
        }
        weakFragment = WeakReference(fragment)
        pictureSelector = PictureSelector.create(fragment)
    }

    fun obtainConfig(): Config {
        return if (config == null) Config() else config!!
    }

    fun setConfig(config: Config): MediaPicker {
        this.config = config
        return this
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_CREATE)
    private fun onCreate() {
        val activity = weakActivity?.get()
        if (activity != null) {
            return
        }

        val fragment = weakFragment?.get()
        if (fragment != null) {

        }
    }

    fun setOnResultCallback(callback: OnResultCallbackListener<LocalMedia>): MediaPicker {
        this.resultCallback = callback
        return this
    }

    fun setSimpleResultCallback(callback: OnSimplePickResultCallback): MediaPicker {
        this.simpleCallback = callback
        return this
    }

    /**
     * 选择
     */
    fun pickImage() {
        openGallery(PictureMimeType.ofImage())
    }

    fun pickVideo() {
        openGallery(PictureMimeType.ofVideo())
    }

    fun pickMedia() {
        openGallery(PictureMimeType.ofAll())
    }

    fun openGallery(type: Int) {
        selectionModel = pictureSelector?.openGallery(type)
        if (config != null) {
            doConfig(config!!)
        }
        selectionModel?.forResult(object : OnResultCallbackListener<LocalMedia?> {
            override fun onResult(p0: MutableList<LocalMedia?>?) {
                this@MediaPicker.onResult(p0)
            }

            override fun onCancel() {
                resultCallback?.onCancel()
                callback?.onCancel()
                simpleCallback?.onCancel()
            }
        })
    }

    /**
     * 拍摄
     */
    fun capturePhoto() {
        openCamera(PictureMimeType.ofImage())
    }

    fun captureVideo() {
        openCamera(PictureMimeType.ofVideo())
    }

    fun openCamera(type: Int) {
        selectionModel = pictureSelector?.openCamera(type)
        if (config != null) {
            doConfig(config!!)
        }
        selectionModel?.forResult(object : OnResultCallbackListener<LocalMedia?> {
            override fun onResult(p0: MutableList<LocalMedia?>?) {
                this@MediaPicker.onResult(p0)
            }

            override fun onCancel() {
                resultCallback?.onCancel()
                callback?.onCancel()
                this@MediaPicker.simpleCallback?.onCancel()
            }
        })
    }

    /**
     * 之前选择的内容，并且不期望选择同样的内容
     * 使用此方式可能需要额外的业务处理
     */
    fun withSelected(preSelectedUri: List<String>?, callback: OnPickResultCallback?): MediaPicker {
        if (preSelectedUri == null) return this
        val result = selectedData.filter {
            it.realPath in preSelectedUri
        }
        selectedData.clear()
        selectedData.addAll(result)

        if (callback != null) {
            this.callback = callback
        }
        return this
    }

    fun withSelected(preSelectedUri: List<String>?): MediaPicker {
        return withSelected(preSelectedUri, null)
    }

    /**
     * 如果不能选择重复的媒体，使用这个回调
     */
    fun setCallback(callback: OnPickResultCallback?): MediaPicker {
        this.callback = callback
        return this
    }

//    fun getSelectData(): List<LocalMedia> {
//        return selectedData
//    }

    fun onResult(p0: MutableList<LocalMedia?>?) {
        val result = p0?.filterNotNull()

        result?.let {
            resultCallback?.onResult(result)

            if (selectedData.isNotEmpty()) {
                //原始选择的数据
                val oriSelect = ArrayList<LocalMedia>(selectedData)

                //选择的最新数据
                val selects =
                    result.filter { r ->
                        r.realPath !in
                                selectedData.map { sd ->
                                    sd.realPath
                                }
                    }
                callback?.onNewSelected(selects)
                //之前选中之后此次又反选的数据
                val unSelects =
                    oriSelect.filter { r ->
                        r.realPath !in result.map {s ->
                            s.realPath
                        }
                    }

                selectedData.addAll(selects)
                unSelects.forEach {us ->
                    selectedData.remove(us)
                }
                callback?.onUnSelected(unSelects)
            } else {
                selectedData.addAll(result)
                callback?.onNewSelected(result)
            }
            this@MediaPicker.callback?.onResult(it)

            if (result.isNotEmpty()) {
                this@MediaPicker.simpleCallback?.onResult(result[0])
            }
        }
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_DESTROY)
    private fun release() {
        var context = weakActivity?.get()
        if (config == null) {
            val wf = weakFragment?.get()
            if (wf != null) {
                context = wf.activity
            }
            if (context != null) {
                PictureCacheManager.deleteAllCacheDirRefreshFile(context);
            }
        }
        selectedData.clear()
        weakFragment?.clear()
        weakActivity?.clear()
    }

    private fun doConfig(config: Config) {
        if (selectionModel == null) {
            return
        }

        var context = weakActivity?.get()
        if (config == null) {
            val wf = weakFragment?.get()
            if (wf != null) {
                context = wf.activity
            }
        }

        selectionModel!!.imageEngine(GlideEngine.createGlideEngine())// 外部传入图片加载引擎，必传项
            .setPictureUIStyle(PictureSelectorUIStyle.ofDefaultStyle())
            //.setPictureStyle(mPictureParameterStyle)// 动态自定义相册主题
            //.setPictureCropStyle(mCropParameterStyle)// 动态自定义裁剪主题
            .setPictureWindowAnimationStyle(PictureWindowAnimationStyle.ofCustomWindowAnimationStyle(R.anim.picture_anim_up_in, R.anim.picture_anim_down_out))// 自定义相册启动退出动画
            .isWeChatStyle(false)// 是否开启微信图片选择风格
            .isUseCustomCamera(true)// 是否使用自定义相机
            .setLanguage(LanguageConfig.CHINESE)// 设置语言，默认中文
            .isPageStrategy(true)// 是否开启分页策略 & 每页多少条；默认开启
            .setRecyclerAnimationMode(AnimationType.DEFAULT_ANIMATION)// 列表动画效果
            .isWithVideoImage(true)// 图片和视频是否可以同选,只在ofAll模式下有效
            //.isSyncCover(true)// 是否强制从MediaStore里同步相册封面，如果相册封面没显示异常则没必要设置
            //.isCameraAroundState(false) // 是否开启前置摄像头，默认false，如果使用系统拍照 可能部分机型会有兼容性问题
            .isCameraRotateImage(true) // 拍照图片旋转是否自动纠正
            //.isAutoRotating(false)// 压缩时自动纠正有旋转的图片
            .isMaxSelectEnabledMask(true)// 选择数到了最大阀值列表是否启用蒙层效果
            //.isAutomaticTitleRecyclerTop(false)// 连续点击标题栏RecyclerView是否自动回到顶部,默认true
            //.loadCacheResourcesCallback(GlideCacheEngine.createCacheEngine())// 获取图片资源缓存，主要是解决华为10部分机型在拷贝文件过多时会出现卡的问题，这里可以判断只在会出现一直转圈问题机型上使用
            //.setOutputCameraPath(createCustomCameraOutPath())// 自定义相机输出目录
//            .setCustomCameraFeatures(CustomCameraView.BUTTON_STATE_BOTH)// 设置自定义相机按钮状态
//            .setCaptureLoadingColor(ContextCompat.getColor(getContext(), R.color.app_color_blue))
            .maxSelectNum(config.maxSelectNum)// 最大图片选择数量
            .minSelectNum(config.minSelectNum)// 最小选择数量
            .maxVideoSelectNum(config.maxSelectNum) // 视频最大选择数量
            .minVideoSelectNum(config.minSelectNum)// 视频最小选择数量
            //.closeAndroidQChangeVideoWH(!SdkVersionUtils.checkedAndroid_Q())// 关闭在AndroidQ下获取图片或视频宽高相反自动转换
            .imageSpanCount(4)// 每行显示个数
            //.queryFileSize() // 过滤最大资源,已废弃
            .filterMinFileSize(if (config.minFileSize.toLong() >= 1048576L) config.minFileSize.toLong() else config.minFileSize / 1024L)// 过滤最小资源
            .filterMaxFileSize(if (config.maxFileSize.toLong() >= 1048576L) config.maxFileSize.toLong() else config.maxFileSize / 1024L)// 过滤最大资源
            .isReturnEmpty(true)// 未选择数据时点击按钮是否可以返回
//            .closeAndroidQChangeWH(true)//如果图片有旋转角度则对换宽高,默认为true
//            .closeAndroidQChangeVideoWH(!SdkVersionUtils.checkedAndroid_Q())// 如果视频有旋转角度则对换宽高,默认为false
            .isAndroidQTransform(true)// 是否需要处理Android Q 拷贝至应用沙盒的操作，只针对compress(false); && .isEnableCrop(false);有效,默认处理
//            .setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT)// 设置相册Activity方向，不设置默认使用系统
            .isOriginalImageControl(config.isOriginalControl)// 是否显示原图控制按钮，如果设置为true则用户可以自由选择是否使用原图，裁剪功能将会失效
            .isDisplayOriginalSize(true)// 是否显示原文件大小，isOriginalImageControl true有效
            .isEditorImage(config.isEditorImage)//是否编辑图片
            .isAutoScalePreviewImage(true)// 如果图片宽度不能充满屏幕则自动处理成充满模式
            //.bindCustomPlayVideoCallback(new MyVideoSelectedPlayCallback(getContext()))// 自定义视频播放回调控制，用户可以使用自己的视频播放界面
            //.bindCustomPreviewCallback(new MyCustomPreviewInterfaceListener())// 自定义图片预览回调接口
            //.bindCustomCameraInterfaceListener(new MyCustomCameraInterfaceListener())// 提供给用户的一些额外的自定义操作回调
            //.bindCustomPermissionsObtainListener(new MyPermissionsObtainCallback())// 自定义权限拦截
            //.bindCustomChooseLimitListener(new MyChooseLimitCallback()) // 自定义选择限制条件Dialog
            //.cameraFileName(System.currentTimeMillis() +".jpg")    // 重命名拍照文件名、如果是相册拍照则内部会自动拼上当前时间戳防止重复，注意这个只在使用相机时可以使用，如果使用相机又开启了压缩或裁剪 需要配合压缩和裁剪文件名api
            //.renameCompressFile(System.currentTimeMillis() +".jpg")// 重命名压缩文件名、 如果是多张压缩则内部会自动拼上当前时间戳防止重复
            //.renameCropFileName(System.currentTimeMillis() + ".jpg")// 重命名裁剪文件名、 如果是多张裁剪则内部会自动拼上当前时间戳防止重复
            .selectionMode(if (config.isMutableSelect) PictureConfig.MULTIPLE else PictureConfig.SINGLE)// 多选 or 单选
            .isSingleDirectReturn(false)// 单选模式下是否直接返回，PictureConfig.SINGLE模式下有效
            .isPreviewImage(true)// 是否可预览图片
            .isPreviewVideo(true)// 是否可预览视频
            //.querySpecifiedFormatSuffix(PictureMimeType.ofJPEG())// 查询指定后缀格式资源
            //.queryMimeTypeConditions(PictureMimeType.ofWEBP())
            .isEnablePreviewAudio(true) // 是否可播放音频
            .isCamera(config.canTakePhoto)// 是否显示拍照按钮
            //.isMultipleSkipCrop(false)// 多图裁剪时是否支持跳过，默认支持
            //.isMultipleRecyclerAnimation(false)// 多图裁剪底部列表显示动画效果
            .isZoomAnim(true)// 图片列表点击 缩放效果 默认true
            //.imageFormat(PictureMimeType.PNG) // 拍照保存图片格式后缀,默认jpeg,Android Q使用PictureMimeType.PNG_Q
            .setCameraImageFormat(PictureMimeType.JPEG) // 相机图片格式后缀,默认.jpeg
            .setCameraVideoFormat(PictureMimeType.MP4)// 相机视频格式后缀,默认.mp4
            .setCameraAudioFormat(PictureMimeType.AMR)// 录音音频格式后缀,默认.amr
            .isEnableCrop(config.isCropEnable)// 是否裁剪
//            .basicUCropConfig()//对外提供所有UCropOptions参数配制，但如果PictureSelector原本支持设置的还是会使用原有的设置
            .isCompress(false)// 是否压缩
            //.compressFocusAlpha(true)// 压缩时是否开启透明通道
            //.compressEngine(ImageCompressEngine.createCompressEngine()) // 自定义压缩引擎
            //.compressQuality(80)// 图片压缩后输出质量 0~ 100
            .synOrAsy(false)//同步true或异步false 压缩 默认同步
            //.queryMaxFileSize(10)// 只查多少M以内的图片、视频、音频  单位M
            //.compressSavePath(getPath())//压缩图片保存地址
            //.sizeMultiplier(0.5f)// glide 加载图片大小 0~1之间 如设置 .glideOverride()无效 注：已废弃
            //.glideOverride(160, 160)// glide 加载宽高，越小图片列表越流畅，但会影响列表图片浏览的清晰度 注：已废弃
//            .withAspectRatio(aspect_ratio_x, aspect_ratio_y)// 裁剪比例 如16:9 3:2 3:4 1:1 可自定义
            .hideBottomControls(false)// 是否显示uCrop工具栏，默认不显示
            .isGif(false)// 是否显示gif图片
            .isWebp(false)// 是否显示webp图片,默认显示
            .isBmp(false)//是否显示bmp图片,默认显示
//            .freeStyleCropEnabled(false)// 裁剪框是否可拖拽
            .freeStyleCropMode(OverlayView.DEFAULT_CROP_GRID_ROW_COUNT)// 裁剪框拖动模式
            .isCropDragSmoothToCenter(true)// 裁剪框拖动时图片自动跟随居中
//            .circleDimmedLayer(cb_crop_circular.isChecked())// 是否圆形裁剪
            //.setCropDimmedColor(ContextCompat.getColor(getContext(), R.color.app_color_white))// 设置裁剪背景色值
            //.setCircleDimmedBorderColor(ContextCompat.getColor(getApplicationContext(), R.color.app_color_white))// 设置圆形裁剪边框色值
            //.setCircleStrokeWidth(3)// 设置圆形裁剪边框粗细
            .showCropFrame(true)// 是否显示裁剪矩形边框 圆形裁剪时建议设为false
            .showCropGrid(true)// 是否显示裁剪矩形网格 圆形裁剪时建议设为false
            .isOpenClickSound(false)// 是否开启点击声音
            .selectionData(selectedData.filter { it.realPath != null })// 是否传入已选图片
            //.isDragFrame(false)// 是否可拖动裁剪框(固定)
            //.videoMinSecond(10)// 查询多少秒以内的视频
            //.videoMaxSecond(15)// 查询多少秒以内的视频
            .recordVideoSecond(config.maxVideoDuration / 1000)//录制视频秒数 默认60s
            .isPreviewEggs(true)// 预览图片时 是否增强左右滑动图片体验(图片滑动一半即可看到上一张是否选中)
            //.cropCompressQuality(90)// 注：已废弃 改用cutOutQuality()
//            .cutOutQuality(90)// 裁剪输出质量 默认100
            //.cutCompressFormat(Bitmap.CompressFormat.PNG.name())//裁剪图片输出Format格式，默认JPEG
//            .minimumCompressSize(100)// 小于多少kb的图片不压缩
            //.cropWH()// 裁剪宽高比，设置如果大于图片本身宽高则无效
            //.cropImageWideHigh()// 裁剪宽高比，设置如果大于图片本身宽高则无效
            .rotateEnabled(false) // 裁剪是否可旋转图片
            .scaleEnabled(true)// 裁剪是否可放大缩小图片
            .videoQuality(1)// 视频录制质量 0 or 1


        selectionModel!!.apply {
            if (config.aspectRatio != null) {
                withAspectRatio(config.aspectRatio!!.ratioX, config.aspectRatio!!.ratioY)
            }
        }
    }

    /*private fun isSupportFormat(filepath: String): Boolean {
        return (filepath.endsWith("jpg") || filepath.endsWith("jpeg")
                || filepath.endsWith("png") || filepath.endsWith("svg")
                || filepath.endsWith("mp4"))
    }*/


}

//只需要选择一个内容时可使用此类
abstract class OnSimplePickResultCallback {

    open fun onResult(medias: LocalMedia) {
    }

    open fun onCancel() {
    }

}

abstract class OnPickResultCallback: OnResultCallbackListener<LocalMedia> {

    //反选/不选择
    open fun onUnSelected(medias: List<LocalMedia>) {}

    //这个只回调最新选择的
    open fun onNewSelected(medias: List<LocalMedia>){}

    override fun onResult(medias: List<LocalMedia>) {
    }

    override fun onCancel() {

    }

}